#include "app.h"

Helper::Helper()
{

}

QString Helper::fileHash(const QString &fileName, QCryptographicHash::Algorithm hashAlgorithm){
    QFile file(fileName);
    if (file.open(QIODevice::ReadOnly)) {
        QByteArray fileData = file.readAll();
        QByteArray hashData = QCryptographicHash::hash(fileData, hashAlgorithm);
        return hashData.toHex();
    }
    return QByteArray();
}

int Helper::getVersion(QString string) {
    std::string str(string.toLatin1().data());
    int version = 0;
    for (unsigned i = str.find("_v") + 2; i < str.find("_v") + 7; i += 2) {
        version = version * 10 + (str[i] - '0');
    }
    return version;
}

void Helper::checkTab(QString tabname){
    App *app = &App::getInstance();
    QList<QGroupBox *> boxes = app->window->findChildren<QGroupBox *> ();
    foreach (QGroupBox * box, boxes){
        if(box->parent()->objectName() == "centralWidget"){
            QLabel* boximage = app->window->findChild<QLabel*>(box->objectName() + "Image");
            if(box->objectName() == tabname){
                box->show();
                if(boximage != nullptr) boximage->show();
            } else {
                box->hide();
                if(boximage != nullptr) boximage->hide();
            }
        }
    }
    app->window->ui->serversbox->show();
}

void Helper::loadNews(){
    App *app = &App::getInstance();
    app->logSectionStart("Загружаем скин лаунчера");
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Загрузка новостей с сайта");
    QString response = app->network->query(app->config->getValue("Network", "news"));
    if(response == "error"){
        qInfo("%s:%i: %s", __FILE__, __LINE__, "Отсутствует связь с сервером. Прервано.");
        app->logSectionEnd();
        return;
    }
    QStringList data = response.split(":::");
    qInfo("%s:%i: %s%s", __FILE__, __LINE__, "Получено новостей - ", app->helper->stringToChar(QString::number(data.count())));
    int counter = 1;
    foreach(QString news, data){
        qInfo("%s:%i: %s%s", __FILE__, __LINE__, "Начинаем размещение новости №", app->helper->stringToChar(QString::number(counter)));
        QStringList item = news.split("|");
        QLabel* image = app->window->ui->newsbox->findChild<QLabel*>("news" + QString::number(counter) + "Image");
        QLabel* title = app->window->ui->newsbox->findChild<QLabel*>("news" + QString::number(counter) + "Title");
        QLabel* teaser = app->window->ui->newsbox->findChild<QLabel*>("news" + QString::number(counter) + "Teaser");
        qInfo("%s:%i: %s", __FILE__, __LINE__, "Устанавливаем миниатюру новости");
        QByteArray bytes = app->network->query_binary(item[0]);
        QImage img(36, 36, QImage::Format_Indexed8);
        img.loadFromData(bytes);
        QString skin = app->helper->getSkinName();
        if(img.bits() == nullptr){
            qInfo("%s:%i: %s", __FILE__, __LINE__, "Миниатюра не найдена, устанавливаем заглушку");
            img.load(":/res/styles/" + skin + "/nophoto.jpg");
        }
        image->setPixmap(QPixmap::fromImage(img));
        qInfo("%s:%i: %s", __FILE__, __LINE__, "Применяем HTML-цвета");
        QString hrefcolor = app->helper->htmlColors.contains("a") ? app->helper->htmlColors.lowerBound("a").value() : "blue";
        QString html = "<a href=\"" + item[3] +"\" style=\"color:" + hrefcolor + "; text-decoration:none;\">" + item[1] + "</a>";
        qInfo("%s:%i: %s", __FILE__, __LINE__, "Размещаем тексты");
        title->setText(html);
        teaser->setText(item[2]);
        counter = counter+1;
    }
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Выполнено.");
    app->logSectionEnd();
}

void Helper::loadHelp(){
    App *app = &App::getInstance();
    app->logSectionStart("Создание справки");
    QFile file(QApplication::applicationDirPath() + "/readme.html");

    if(file.open(QFile::ReadOnly | QFile::Text)){
        qInfo("%s:%i: %s", __FILE__, __LINE__, "Справка загружена. Устанавливаем HTML-цвета и выводим.");
        app->window->ui->textBrowser->setHtml("");
        QString content = file.readAll();

        app->stylehtml = "<style>";
        for(auto e : app->helper->htmlColors.keys()){
            app->stylehtml = app->stylehtml + e + "{color:" + app->helper->htmlColors.value(e) + ";}";
        }


        QFile styleF;
        QString skin = app->helper->getSkinName();
        styleF.setFileName(app->app_folder + "styles/" + skin + "/readme.css");
        if(styleF.open(QFile::ReadOnly)){
           app->stylehtml = "<html><head><style type=\"text/css\">" + app->stylehtml + styleF.readAll();
        }

        app->stylehtml = app->stylehtml + "</style></head>";

        app->window->ui->textBrowser->setHtml(app->stylehtml + content);
        app->window->ui->textBrowser->document()->setDefaultStyleSheet(styleF.readAll());
        qInfo("%s:%i: %s", __FILE__, __LINE__, "Выполнено.");
        app->logSectionEnd();
    } else {
        qWarning("%s:%i: %s", __FILE__, __LINE__, "Не удалось получить информацию с сервера");
        qInfo("%s:%i: %s", __FILE__, __LINE__, "Выполнено.");
        app->logSectionEnd();
    }
}

void Helper::hideAll(){
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Прячем ненужные элементы.");
    App *app = &App::getInstance();
    app->window->ui->optbox->hide();
    app->window->ui->optboxImage->hide();
    app->window->ui->newsbox->hide();
    app->window->ui->newsboxImage->hide();
    app->window->ui->helpbox->hide();
    app->window->ui->helpboxImage->hide();
    app->window->ui->aboutbox->hide();
    app->window->ui->aboutboxImage->hide();
    app->window->ui->dialogbox->hide();
    app->window->ui->loader->hide();
    app->window->ui->dialogList->hide();
    app->window->ui->startbox->hide();    
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Устанавливаем всплывающие подсказки.");
    app->window->ui->goOriginal->setToolTip("Запуск оригинальной версии игры");
    app->window->ui->goLocal->setToolTip("Запуск русской версии игры");
    app->window->ui->goCancel->setToolTip("Вернуться обратно");
    app->window->ui->goProgress->hide();
    app->window->ui->goHint->hide();
    app->window->ui->footRed->hide();
    app->window->ui->footYellow->hide();
}

void Helper::setAbout(){
    App *app = &App::getInstance();
    app->logSectionStart("О нас");
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Размещаем информацию о хороших людях.");
    app->window->ui->programmerslist->setText("Gi1dor, coder");
    app->window->ui->designerslist->setText("Чирик, valyot, Quilanar, Eodoro");
    app->window->ui->translatorslist->setText("Arissa, unnamedartist, valyot, Чирик, Eodoro, Ferdik, Tukbard, Alcassar, Satenskyhou, Quilanar, Коралан, Invizet, taiki_exe, Rhymer, Takhizis, Lauraldo, seventhgate027, Kerita, Хилко, Leodgar, Juolukka, Auruman, Drazgar, Tondred, Foldoren, Exildil, Anormir, Василий Волков, Andoriel, Felicer, noldohini, Gi1dor, Progylkin, Arkwendir, Bara Norkins, Siltier, Maxsikr, Nahkrin, Nick_os, Razverg, ENDrain, ToX4SHOW, Jeyolin, Frenni, Коралан, Palmuch, Marty");
    app->window->ui->programmerslist->setText("Gi1dor, coder");
    app->window->ui->helperslist->setText("Menelhad, Мурзилка, Pastor, Alvees, Mirta, иван петров, vensys, Arkelas, Чеширская Улыбка, Shanishka70 ");
}

void Helper::findSkins(){
    App *app = &App::getInstance();
    app->logSectionStart("Темы оформления");
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Производим поиск доступных тем.");
    QDir dir(QApplication::applicationDirPath() + "/styles");
    QStringList skins = dir.entryList(QDir::Dirs | QDir::AllDirs | QDir::NoDotAndDotDot);
    app->window->ui->skinSelect->addItems(skins);
    QString skinname = app->config->getValue("Global", "current_theme");
    app->window->ui->skinSelect->setCurrentText(skinname);

    qInfo("%s:%i: %s", __FILE__, __LINE__, "Устанавливаем отметку о файле локализации.");
    QString file = app->config->getValue("Local", "file");
    if(file == "client_local_English.dat") app->window->ui->enButton->setEnabled(false);
    if(file == "client_local_DE.dat") app->window->ui->deButton->setEnabled(false);
    if(file == "client_local_FR.dat") app->window->ui->frButton->setEnabled(false);
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Выполнено.");
    app->logSectionEnd();
}

QString Helper::getSkinName() {
    App *app = &App::getInstance();
    QString skin = app->config->getValue("Global", "current_theme");
    if(skin == "-1") skin = app->config->getValue("Global", "default_theme");
    qInfo("%s:%i: %s%s", __FILE__, __LINE__, "Текущая тема оформления: ", app->helper->stringToChar(skin));
    return skin;
}

bool Helper::fileExists(QString path) {
    App *app = &App::getInstance();
    QFileInfo check_file(path);
    bool exists = check_file.exists() && check_file.isFile();
    if(exists == false) qWarning("%s:%i: %s%s", __FILE__, __LINE__, "Файл не найден: ", app->helper->stringToChar(path));
    return exists;
}

void Helper::applyShadow(QWidget * obj, QString color, int blur, int offset_x, int offset_y){
    QGraphicsDropShadowEffect *pLabelTextShadowEffect = new QGraphicsDropShadowEffect(obj);
    pLabelTextShadowEffect->setColor(QColor(color));
    pLabelTextShadowEffect->setBlurRadius(blur);
    pLabelTextShadowEffect->setOffset(offset_x, offset_y);
    obj->setGraphicsEffect(pLabelTextShadowEffect );

}

void Helper::myDialogBox(QString text, QString info, QString okText, QString cancelText, QString image, QString okAnswer, QString cancelAnswer,
                        int w, int h, bool textshadow = false, bool is_cancel = false){

    App *app = &App::getInstance();

    if(okAnswer == "") okAnswer = "close";
    app->helper->dialog_ok_answer = okAnswer;
    if(cancelAnswer == "") cancelAnswer = "close";
    app->helper->dialog_cancel_answer = cancelAnswer;

    app->window->ui->okButton->setText(okText);
    app->window->ui->cancelButton->setText(cancelText);
    app->window->ui->dialogbox->setGeometry(0, 0, app->window->width(), app->window->height());

    QSettings settings( app->config->getSkinFile(), QSettings::IniFormat );
    settings.beginGroup("mainWindow");
    int left = (settings.value("width").toInt() - w)/2;
    int top = (settings.value("height").toInt() - h)/2;
    app->window->ui->dialog->setGeometry(QRect(left, top, w, h));
    settings.endGroup();

    int image_height = app->window->ui->dialog->rect().height() - app->window->ui->dialogHeader->rect().height();
    app->window->ui->dialogImage->setGeometry(0,
        app->window->ui->dialogHeader->rect().bottom(),
        image_height/2 ,image_height
     );

    QString skin = app->helper->getSkinName();
    QString iconpath = "";
    iconpath = app->helper->fileExists("styles/" + skin + "/" + image) ? "styles/" + skin + "/" + image : ":/res/styles/" + skin + "/" + image;
    QPixmap icon(iconpath);

    app->window->ui->dialogImage->setPixmap(icon.scaled(app->window->ui->dialogImage->width(), app->window->ui->dialogImage->height()));
    if(icon.isNull()){
        app->window->ui->dialogImage->setGeometry(QRect(10, 10, 10, 10));
    }

    app->window->ui->dialogHeader->setGeometry(
                app->window->ui->dialogHeader->rect().left(),
                app->window->ui->dialogHeader->rect().top(),
                app->window->ui->dialog->rect().width(),
                app->window->ui->dialogHeader->rect().height()
    );

    app->window->ui->dialogInfo->setGeometry(app->window->ui->dialogImage->rect().right(),
        app->window->ui->dialogHeader->rect().bottom(),
         app->window->ui->dialog->rect().width() - app->window->ui->dialogImage->rect().width()-7,
         image_height - app->window->ui->cancelButton->height()
     );



    if(is_cancel){
       app->window->ui->cancelButton->setGeometry(
           app->window->ui->dialog->rect().left() + app->window->ui->dialog->width() - app->window->ui->cancelButton->width() - 20,
           app->window->ui->dialog->rect().bottom() - app->window->ui->cancelButton->height() - 10,
           app->window->ui->cancelButton->width(),
           app->window->ui->cancelButton->height()
        );
       app->window->ui->okButton->setGeometry(
           app->window->ui->dialog->rect().left() + app->window->ui->dialog->width() - app->window->ui->cancelButton->width() - app->window->ui->cancelButton->width() - 40,
           app->window->ui->dialog->rect().bottom() - app->window->ui->cancelButton->height() - 10,
           app->window->ui->cancelButton->width(),
           app->window->ui->cancelButton->height()
        );
    } else {
        app->window->ui->cancelButton->setGeometry(-1000,-1000, app->window->ui->cancelButton->width(), app->window->ui->cancelButton->height());
        app->window->ui->okButton->setGeometry(
            app->window->ui->dialog->rect().left() + app->window->ui->dialog->width() - app->window->ui->cancelButton->width() - 20,
            app->window->ui->dialog->rect().bottom() - app->window->ui->cancelButton->height() - 10,
            app->window->ui->cancelButton->width(),
            app->window->ui->cancelButton->height()
         );
    }

    app->window->ui->dialogHeader->setText(text);
    app->window->ui->dialogInfo->setText(info);
    app->window->ui->dialogImage->setText("");
    app->window->ui->dialogbox->show();
}

void Helper::setState(QString state){
    App *app = &App::getInstance();
    if(state == "busy"){
        app->window->ui->closeButton->setEnabled(false);
        app->window->ui->enterButton->setEnabled(false);
        app->window->ui->lotropathButton->setEnabled(false);
        app->window->ui->repairButton->setEnabled(false);
        app->state = "busy";
        app->window->ui->loader->show();
    }

    if(state == "free"){
        app->window->ui->closeButton->setEnabled(true);
        app->window->ui->enterButton->setEnabled(true);
        app->window->ui->lotropathButton->setEnabled(true);
        app->window->ui->repairButton->setEnabled(true);
        app->state = "free";
        app->window->ui->loader->hide();
    }

    if(state == "nofolder"){
        app->window->ui->enterButton->setEnabled(false);
        app->state = "nofolder";
        app->window->ui->loader->hide();
    }

    if(state == "runfirst"){
        app->window->ui->enterButton->setEnabled(false);
        app->window->ui->repairButton->setEnabled(false);
        app->state = "runfirst";
        app->window->ui->loader->hide();
        QString info = "Вы запускаете программу русификации LotRO впервые. Мастер установки поможет вам настроить программу для ее дальнейшего использования. Сейчас будет осуществлена попытка автоматически определить каталог с игрой. Для продолжения нажмите «Далее».";
        app->helper->myDialogBox("Добро пожаловать!", info, "Далее", "Отмена", "gandalf.png", "close gamefind", "", 400, 200, true, false);
    }
}

int Helper::dateToTimestamp(QString customDateString, QString format){
    int timestamp = QDateTime::fromString(customDateString, format).toTime_t();
    return timestamp;
}

void Helper::installMicroPatch(){
    App *app = &App::getInstance();
    if(app->config->getValue("Updates", "micro") != "true") return;
    app->helper->setState("busy");
    app->window->ui->hintLabel->setText("Установка последних обновлений");
    app->window->ui->exthintLabel->setText("Идет проверка и установка новых обновлений");
    QFuture<void> future = QtConcurrent::run([=]() {
        app->helper->prepareMicroPatch();
        app->helper->applyMicroPatch();
        QDir dir("/data/micro");
        app->helper->clearFolder(dir);
    });
    app->window->ui->hintLabel->setText("Установка завершена");
    app->window->ui->exthintLabel->setText("Обновления успешно установлены");
    app->helper->setState("free");

}

void Helper::prepareMicroPatch(){
    App *app = &App::getInstance();
    app->logSectionStart("Автоматическая проверка обновлений");
    app->helper->setState("busy");
    app->window->ui->hintLabel->setText("Загрузка последних обновлений");
    app->window->ui->exthintLabel->setText("Подождите пока загрузятся последние изменения в системе переводов");
    QString date = app->config->getValue("Updates", "update");

    if (date == "false" || date == "-1")
        date = QDate::currentDate().toString("yyyy-MM-dd");

    QTime time = QTime::currentTime();
    int seconds = QTime(0, 0, 0).secsTo(time);
    int timestamp = app->helper->dateToTimestamp(date + " " + time.toString("hh:mm:ss"), "yyyy-MM-dd hh:mm:ss") - 2592000;

    qInfo("%s:%i: %s", __FILE__, __LINE__, "Опция активна. Начинаем загрузку обновлений");
    app->network->micropatch = QApplication::applicationDirPath() + "/data/micro/" + QString::number(timestamp + seconds) + ".db";
    if(!app->helper->fileExists(app->network->micropatch)){
        qInfo("%s:%i: %s%s", __FILE__, __LINE__, "Файл патча: ", app->helper->stringToChar(app->network->micropatch));
        app->network->micropatch = app->network->getMicroPath(timestamp);
    }
    app->window->ui->hintLabel->setText("Загрузка завершена");
    app->window->ui->exthintLabel->setText("Все обновления были успешно загружены и доступны к установке");

    app->helper->setState("free");
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Выполнено.");
    app->logSectionEnd();
}

void Helper::applyMicroPatch(){
    App *app = &App::getInstance();
    app->logSectionStart("Установка мгновенных обновлений");
    app->helper->setState("busy");
    app->window->ui->hintLabel->setText("Установка последних обновлений");
    app->window->ui->exthintLabel->setText("Подождите пока закончится применение обновлений");

    app->db->InitDatabase(app->helper->stringToChar(app->network->micropatch));
    int indb = app->db->CountRows();
    qInfo("%s:%i: %s%d", __FILE__, __LINE__, "Файлов в обновлении: ", indb);
    for(int i = 0; i<=indb; i++){ processFile(); }
    //for (int i = 0; i < 5; i++) app->datfiles[i].CommitChanges();
    app->db->CloseDatabase();

    app->helper->setState("free");
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Выполнено.");
    app->logSectionEnd();
}

void Helper::applyLoadscreens(){
    App *app = &App::getInstance();
    app->logSectionStart("Установка заставочных экранов");
    app->helper->setState("busy");

    QString datafolder = QApplication::applicationDirPath() + "/data";
    QDir dir(datafolder);
    if (!dir.exists())
        return;
    QStringList paths = dir.entryList(QStringList("loadscreens*"));

    if(!paths.empty()){

        if(paths.first() == app->config->getValue("Applied", "screens")){
            app->helper->setState("free");
            return;
        }

        QString lang = app->config->getValue("Local", "lang");

        if (lang == "-1") {
            app->config->setValue("Local", "lang", "en");
            lang = "en";
        }

        QString folder = app->config->getValue("Local", "folder") + "raw/" + lang + "/logo/";
        SubfileData subfile;
        QStringList filenames;
        QString mainscreen = lang == "en" ? "lotro_ad_pregame.jpg" : "lotro_ad_pregame_" + lang + ".jpg";
        filenames << mainscreen << "lotro_generic_teleport_screen_01.jpg" << "lotro_generic_teleport_screen_02.jpg"
                  << "lotro_generic_teleport_screen_03.jpg" << "lotro_generic_teleport_screen_04.jpg" << "lotro_generic_teleport_screen_05.jpg"
                  << "lotro_generic_teleport_screen_06.jpg" << "lotro_generic_teleport_screen_07.jpg" << "lotro_generic_teleport_screen_08.jpg"
                  << "lotro_generic_teleport_screen_09.jpg" << "lotro_generic_teleport_screen_10.jpg";

        QString basename = datafolder + "/" + paths.first();
        app->db->InitDatabase(basename.toStdString());
        int indb = app->db->CountRows();
        qInfo("%s:%i: %s%d", __FILE__, __LINE__, "Файлы загрузочных экранов: ", indb);
        for(int i = 0; i<indb; i++){
            subfile = app->db->GetNextFile();
            qDebug() << i;
            if (!subfile.Empty()){
                qInfo("%s:%i: %s", __FILE__, __LINE__, app->helper->stringToChar(folder + filenames[i]));
                QFile::remove(folder + filenames[i]);
                subfile.binary_data.WriteToFile((folder + filenames[i]).toStdString());
            }
        }
        app->db->CloseDatabase();

    }
    app->helper->setState("free");
    app->window->ui->loadscreensStatus->setText("Готово");
    app->config->setValue("Applied", "screens", paths.first());
    qInfo("%s:%i: %s", __FILE__, __LINE__, "Выполнено.");
    app->logSectionEnd();
}

void Helper::addComboItem(QComboBox* combo, QString itemName, QIcon* icon){
    combo->insertItem(0, itemName);
    combo->setItemIcon(0, *icon);
}

void Helper::saveLocale(int locale, QPushButton * button){
    App *app = &App::getInstance();
    QString file;

    if(locale == 0) file = "client_local_English.dat";
    if(locale == 1) file = "client_local_DE.dat";
    if(locale == 2) file = "client_local_FR.dat";
    if(app->helper->fileExists(app->config->getValue("Local", "folder") + "/" + file)){
        app->window->ui->enButton->setEnabled(true);
        app->window->ui->deButton->setEnabled(true);
        app->window->ui->frButton->setEnabled(true);
        button->setEnabled(false);
        app->config->setValue("Local", "file", file);
    } else {
        QString text = "Данная локализация отсутствует";
        QString info = "В настоящий момент вы не можете выбрать данную опцию, поскольку файл " + file + " отсутствует в вашем игровом клиенте. Чтобы скачать его, запустите официальный лаунчер, выберите желаемый язык в верхней правой части окна и дождитесь, когда загрузка необходимых файлов завершится.";
        app->helper->myDialogBox(text, info, "OK", "Отмена", "gandalf.png", "", "", 400, 220, true, false);
    }
}

bool Helper::setGameLocale(QString locale){
    App *app = &App::getInstance();
    app->window->ui->goHint->show();
    app->helper->setState("busy");
    app->window->ui->goLocal->setEnabled(false);
    app->window->ui->goOriginal->setEnabled(false);
    app->window->ui->goCancel->setEnabled(false);

    if(app->helper->openDatFile(0)){
        LOCALE loc = app->datfiles[0].current_locale();
        dat_locale = loc;

        if(loc == PATCHED) qDebug() << "Starting " + locale + " version. Current locale is PATCHED";
        if(loc == ORIGINAL) qDebug() << "Starting " + locale + " version. Current locale is ORIGINAL";
        if(loc != ORIGINAL && loc != PATCHED) qDebug() << "Starting " + locale + " version. Current locale is UNKNOWN";

        if(locale == "RU" && loc != PATCHED){ qDebug()<< "Current locale Original"; app->datfiles[0].SetLocale(PATCHED); dat_locale = PATCHED;}
        if(locale == "Original" && loc != ORIGINAL){ qDebug()<< "Current locale RU"; app->datfiles[0].SetLocale(ORIGINAL); dat_locale = ORIGINAL;}
        app->helper->closeDatFile(0);
    }

    app->window->ui->goHint->hide();
    app->window->ui->goProgress->hide();
    app->window->ui->startbox->hide();
    app->window->ui->goLocal->setEnabled(true);
    app->window->ui->goOriginal->setEnabled(true);
    app->window->ui->goCancel->setEnabled(true);
    app->helper->setState("free");
    qDebug() << "Locale changed";

    return true;
}

void Helper::startGame(){
    App *app = &App::getInstance();
    app->config->saveAllDatFiles();
    QStringList args;
    args << "-skiprawdownload" << "-nosplash";
    if (dat_locale == PATCHED)
        args << "-disablePatch";

    QFile f(app->config->getValue("Local", "folder") + "/TurbineLauncher.exe");
    QProcess process;
    if(fileExists(f.fileName())){
        if(f.fileName().contains(" ")) f.setFileName("\"" + f.fileName() + "\"");
        process.startDetached(f.fileName(), args);
        process.waitForFinished(-1);
        process.deleteLater();
        QApplication::quit();
    } else {
        QString text = "Запуск не удался";
        QString info = "Не удалось запустить игру. Во время запуска произшла непредвиденная ошибка (возможно, не найден файл TurbineLauncher.exe в папке с игрой. Проверьте в 'Настройках', что у вас указан верный путь к игре, и повторите попытку запуска).";
        app->helper->myDialogBox(text, info, "OK", "Отмена", "gandalf.png", "", "", 400, 160, true, false);
    }
}

bool Helper::openDatFile(int id){
    App *app = &App::getInstance();
    QStringList dat_files;
    dat_files << app->config->getValue("Local", "file") << "client_general.dat" << "client_sound.dat" << "client_surface.dat" << "client_highres.dat";
    if(dat_files[id] != nullptr) {
        qDebug() << dat_files[id];
        const char * filename = app->helper->stringToChar(app->config->getValue("Local", "folder") + "/" + dat_files[id]);
        std::string file = app->config->getDatPath(0);
        qDebug("%s:%i: %s%s", __FILE__, __LINE__, "Инициализация dat-файла. Открываем файл ", filename);
        int dat_state = app->datfiles[id].InitDatFile(file, 0);
        qDebug("%s:%i: %s%d", __FILE__, __LINE__, "Состояние dat-файла: ", dat_state);
        if(dat_state > 0){
            app->helper->setState("busy");
            return true;
        } else {
            QString text = "Нет доступа к  файлам игры";
            QString info = "Возможно в данный момент запущены клиент или лаунчер игры либо попытка доступа блокируется настройкой прав, антивирусом или другими приложениями. Завершите все процессы, использующие файлы игры и повторите попытку.";
            app->helper->myDialogBox(text, info, "OK", "Отмена", "gandalf.png", "", "", 550, 200, true, false);
            app->helper->setState("free");
            return false;
        }
    }
    return true;
}

void Helper::closeDatFile(int id){
    App *app = &App::getInstance();
    //app->datfiles[id].CommitChanges();
    app->datfiles[id].CloseDatFile();
    app->helper->setState("free");
}

const char * Helper::stringToChar(QString val){
    ba = val.toLocal8Bit();
    const char * result = ba.data();
    return result;
}

void Helper::applyPatch(QString name){
    App *app = &App::getInstance();
    app->window->ui->hintLabel->setText("Установка патчей");
    app->window->ui->exthintLabel->setText("Ожидайте. Идет установка выбранных патчей");
    qDebug("%s:%i: %s%s", __FILE__, __LINE__, "Начинаем применение патча ", app->helper->stringToChar(name));
    QLabel* label = app->window->findChild<QLabel*>(name + "Status");
    QDir dir(QApplication::applicationDirPath() + "/data");
    if (!dir.exists()) {
        app->window->ui->hintLabel->setText("Установка патчей завершена с ошибкой");
        app->window->ui->exthintLabel->setText("Ошибка - не найдена папка data");
        return;
    }

    QStringList paths = dir.entryList(QStringList(name + "*"));
    qDebug() << app->helper->stringToChar("data/" + paths.first());
    if(!paths.empty()){         
       app->db->InitDatabase(app->helper->stringToChar("data/" + paths.first()));
        int indb = app->db->CountRows();
        qInfo("%s:%i: %s%d", __FILE__, __LINE__, "Файлов в обновлении: ", indb);
        for(int i = 0; i<=indb; i++){
            processFile();
            if (label != nullptr) label->setText("Применение ... " + QString::fromLatin1("%1").arg(i*100/indb, 3, 'f', 1) + "%");
        }
        if(label != nullptr)label->setText("Готово");
        //for (int i = 0; i < 5; i++) app->datfiles[i].CommitChanges();
        app->config->setValue("Applied", name, paths.first());
        app->db->CloseDatabase();
    }
    app->window->ui->hintLabel->setText("Установка патчей завершена");
    app->window->ui->exthintLabel->setText("Применение выбранных вами патчей закончено");
}

void Helper::processFile(){
    App *app = &App::getInstance();
    SubfileData subfile;
    subfile = app->db->GetNextFile();
    if (subfile.Empty()){
        qInfo("%s:%i: %s", __FILE__, __LINE__, "Достигнут конец файла.");
    } else {
        int dat_id = subfile.options["did"].as<int>();
        int dat_state = app->datfiles[dat_id].InitDatFile(app->config->getDatPath(dat_id), dat_id);
        //qDebug("%s:%i: %s%d", __FILE__, __LINE__, "Состояние dat-файла: ", dat_state);
        if(dat_state > 0) app->datfiles[dat_id].PatchFile(subfile);
    }
}

void Helper::applyGlobal(){
    App *app = &App::getInstance();

    // Применяем патч с заставочными экранами
    if(app->config->getValue("Editor", "screens") == "true"){
        app->helper->applyLoadscreens();
    } else {
        QString dateline = app->config->getValue("Datetime", "loadscreens");
        if(dateline != "-1"){
            QStringList date = dateline.split(" ");
            app->window->ui->loadscreensStatus->setText(date[0]);
        }
    }

    // Применяем остальные патчи
    QStringList names;
    QStringList need;
    QFuture<void> f;
    f = QtConcurrent::run([=]() {});
    names  << "fonts" << "sounds" << "texts" << "images" << "videos" << "textures";
    QDir dir(QApplication::applicationDirPath() + "/data");

    foreach(QString name, names) {

        QStringList list = dir.entryList(QStringList(name + "*"));

        if (list.size() > 0 && list.first() != ""){
            QString isset = app->config->getValue("Applied", name);
            if (list.first() != isset && app->config->getValue("Editor", name) == "true") {
                need.append(name);
            }
        }
    }

    Helper *helper = new Helper();
    QFutureWatcher<void> * watcher = new QFutureWatcher<void>();
    connect(watcher, SIGNAL(finished()), helper, SLOT(handleFinisheddd()));

    if(need.size() > 0){
        watcher->setFuture(QtConcurrent::run([=]() {
            if(app->helper->openDatFile(0)){
                foreach(QString name, need){ app->helper->applyPatch(name);}
                app->helper->closeDatFile(0);
            }
        }));
    } else {
        app->helper->installMicroPatch();
    }

}

void Helper::handleFinisheddd(){
   App *app = &App::getInstance();
   app->helper->installMicroPatch();
}

bool Helper::isDatReady(){
    App *app = &App::getInstance();
    bool free;
    QString dir = app->config->getValue("Local", "folder");
    QString file = app->config->getValue("Local", "file");
    QFile fl(dir+"/" + file);
    qDebug() << fl.fileName();
    if(app->helper->fileExists(fl.fileName())){
        QDir game_dir(dir);
        free = game_dir.rename(fl.fileName(), fl.fileName() + "99");
        game_dir.rename(fl.fileName() + "99", fl.fileName());
    } else {
        free = false;
    }
    if(free == true) qDebug() << "Dat is FREE"; else qDebug() << "Dat is BUSY";
    return free;
}

void Helper::replaceSkin(QString name){
    App *app = &App::getInstance();
    QString text = "Смена темы оформления";
    QString info = "Выбрана тема " + name + ". Изменения вступят в силу при следующем запуске лаунчера.";
    //app->helper->myDialogBox(text, info, "OK", "Отмена", "gandalf.png", "", "", 400, 120, true, false);

    qInfo("%s:%i: %s", __FILE__, __LINE__, "Оформляем главное окно.");

    QFile styleF;
    QString skin = app->helper->getSkinName();
    styleF.setFileName(app->app_folder + "styles/" + name + "/style.css");
    styleF.open(QFile::ReadOnly);
    QString qssStr = styleF.readAll();

    qApp->setStyleSheet(qssStr);

    QPixmap maskPix(":/res/styles/" + name + "/bg.png");
    QPixmap maskPixLocal(app->app_folder + "styles/" + name + "/bg.png");
    maskPix = maskPix.isNull() ? maskPixLocal : maskPix;
    app->window->setMask(maskPix.mask());

    QString color = app->helper->htmlColors.contains("a") ? app->helper->htmlColors.lowerBound("a").value() : "blue";
    app->config->loadSkin();
    QString hrefcolor = app->helper->htmlColors.contains("a") ? app->helper->htmlColors.lowerBound("a").value() : "blue";

    QString newtext = app->window->ui->news1Title->text().replace(color, hrefcolor);
    app->window->ui->news1Title->setText(newtext);

    newtext = app->window->ui->news2Title->text().replace(color, hrefcolor);
    app->window->ui->news2Title->setText(newtext);

    newtext = app->window->ui->news3Title->text().replace(color, hrefcolor);
    app->window->ui->news3Title->setText(newtext);

    newtext = app->window->ui->news4Title->text().replace(color, hrefcolor);
    app->window->ui->news4Title->setText(newtext);

    app->helper->loadHelp();

    QMovie * movie = new QMovie(":/res/styles/" + skin + "/loader.gif");
    if(!movie->isValid()) movie = new QMovie(app->app_folder + "styles/" + skin + "/loader.gif");
    app->window->ui->loader->setMovie(movie);
    movie->setScaledSize(QSize(app->window->ui->loader->width(), app->window->ui->loader->height()));
    movie->start();
}

int Helper::checkDatFile(){
    //bool CheckIfUpdatedByGame(); // Обновлялся ли игрой
    //bool CheckIfNotPatched(); // Был ли хоть раз пропатчен новым лаунчером
    //bool CheckIfPatchedByOldLauncher(); // Был ли хоть раз пропатчен старым лаунчером

    App *app = &App::getInstance();
    app->helper->openDatFile(0);
    if(app->datfiles[0].CheckIfUpdatedByGame()){
        return 1;
        //app->ready = false;
        //QString text = "Файлы локализации изменены";
        //QString info = "В процессе последнего запуска игры были установлены обновления игрового клиента. Это могло затронуть файлы локализации. Желаете провести проверку и исправить поврежденные участки перевода? Вы можете в любой момент сделать это самостоятельно, нажав кнопку «Применить патчи заново».";
        //app->helper->myDialogBox(text, info, "OK", "Отмена", "gandalf.png", "applypaths", "", 400, 220, true, true);
    }

    if(app->datfiles[0].CheckIfNotPatched() && app->config->getValue("Local", "runfirst") == "1"){
        return 2;
        //app->ready = false;
        //QString text = "Файлы локализации отсутствуют";
        //QString info = "Текущий файл локализации не содержит патчей. Возможно это первый запуск «Наследия» с данным игровым клиентом, или клиент был обновлён игровым лаунчером. Применить выбранные патчи в данному файлу?";
        //app->helper->myDialogBox(text, info, "OK", "Отмена", "gandalf.png", "applypaths", "", 420, 200, true, true);
    }

    if(app->datfiles[0].CheckIfPatchedByOldLauncher()){
        return 3;
        //QString text = "Файлы локализации не являются оригинальными";
        //QString info = "Внимание! Присутствующие в вашем игровом клиенте файлы локализации не являются оригинальными, поскольку были модифицированы другой программой.<br/><strong>Мы настоятельно рекомендуем перекачать оригинальные версии файлов, поскольку правильная работа Наследия с данными файлами не гарантируется!</strong>";
        //app->helper->myDialogBox(text, info, "Перекачать", "Не надо", "gandalf.png", "loaddatfile", "", 550, 200, true, true);
    }
    return 0;
}

void Helper::clearFolder(QDir &dir){
    //Получаем список файлов
    QStringList lstFiles = dir.entryList(QDir::Files);

    //Удаляем файлы
    foreach (QString entry, lstFiles){
        QString entryAbsPath = dir.absolutePath() + "/" + entry;
        //QFile::setPermissions(entryAbsPath, QFile::ReadOwner | QFile::WriteOwner);
        qDebug() << dir.absolutePath();
        QFile::remove(entryAbsPath);
    }
}
